/*
 * Virtio Accessor Support: In case your target can change endian.
 *
 * Copyright IBM, Corp. 2013
 *
 * Authors:
 *  Rusty Russell   <rusty@au.ibm.com>
 *
 * This work is licensed under the terms of the GNU GPL, version 2.  See
 * the COPYING file in the top-level directory.
 *
 */
#ifndef _QEMU_VIRTIO_ACCESS_H
#define _QEMU_VIRTIO_ACCESS_H
#include "hw/virtio/virtio.h"

/* Initialized by virtio_get_byteswap() at any virtio device reset. */
extern bool virtio_byteswap;

static inline uint16_t virtio_lduw_phys(hwaddr pa)
{
    if (virtio_byteswap) {
        return bswap16(lduw_phys(pa));
    }
    return lduw_phys(pa);
    
}

static inline uint32_t virtio_ldl_phys(hwaddr pa)
{
    if (virtio_byteswap) {
        return bswap32(ldl_phys(pa));
    }
    return ldl_phys(pa);
}

static inline uint64_t virtio_ldq_phys(hwaddr pa)
{
    if (virtio_byteswap) {
        return bswap64(ldq_phys(pa));
    }
    return ldq_phys(pa);
}

static inline void virtio_stw_phys(hwaddr pa, uint16_t value)
{
    if (virtio_byteswap) {
        stw_phys(pa, bswap16(value));
    } else {
        stw_phys(pa, value);
    }
}

static inline void virtio_stl_phys(hwaddr pa, uint32_t value)
{
    if (virtio_byteswap) {
        stl_phys(pa, bswap32(value));
    } else {
        stl_phys(pa, value);
    }
}

static inline void virtio_stw_p(void *ptr, uint16_t v)
{
    if (virtio_byteswap) {
        stw_p(ptr, bswap16(v));
    } else {
        stw_p(ptr, v);
    }
}

static inline void virtio_stl_p(void *ptr, uint32_t v)
{
    if (virtio_byteswap) {
        stl_p(ptr, bswap32(v));
    } else {
        stl_p(ptr, v);
    }
}

static inline void virtio_stq_p(void *ptr, uint64_t v)
{
    if (virtio_byteswap) {
        stq_p(ptr, bswap64(v));
    } else {
        stq_p(ptr, v);
    }
}

static inline int virtio_lduw_p(const void *ptr)
{
    if (virtio_byteswap) {
        return bswap16(lduw_p(ptr));
    } else {
        return lduw_p(ptr);
    }
}

static inline int virtio_ldl_p(const void *ptr)
{
    if (virtio_byteswap) {
        return bswap32(ldl_p(ptr));
    } else {
        return ldl_p(ptr);
    }
}

static inline uint64_t virtio_ldq_p(const void *ptr)
{
    if (virtio_byteswap) {
        return bswap64(ldq_p(ptr));
    } else {
        return ldq_p(ptr);
    }
}

static inline uint16_t virtio_tswap16(uint16_t s)
{
    if (virtio_byteswap) {
        return bswap16(tswap16(s));
    } else {
        return tswap16(s);
    }
}

static inline void virtio_tswap16s(uint16_t *s)
{
    tswap16s(s);
    if (virtio_byteswap) {
        *s = bswap16(*s);
    }
}

static inline uint32_t virtio_tswap32(uint32_t s)
{
    if (virtio_byteswap) {
        return bswap32(tswap32(s));
    } else {
        return tswap32(s);
    }
}

static inline void virtio_tswap32s(uint32_t *s)
{
    tswap32s(s);
    if (virtio_byteswap) {
        *s = bswap32(*s);
    }
}
#endif /* _QEMU_VIRTIO_ACCESS_H */
